# Generated by Django 2.1.2 on 2018-11-15 09:26
from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('document', '0002_auto_20180814_1946'),
    ]

    operations = [
        migrations.AlterField(
            model_name='document',
            name='doc_version',
            field=models.DecimalField(decimal_places=1, default=2.3, max_digits=3),
        ),
    ]
