# Generated by Django 2.2.3 on 2019-08-09 14:55

from django.db import migrations


class Migration(migrations.Migration):
    atomic = False
    dependencies = [
        ('document', '0013_auto_20190808_1126'),
        ('style', '0003_auto_20190803_0749'),
    ]

    operations = [
        migrations.RenameModel(
            old_name='DocumentStyle',
            new_name='OldDocumentStyle',
        ),
    ]
