"""
Contains base templates (html),
statics that are shared among different django applications
Also contains helper for starting tornado server and management commands
"""
default_app_config = 'base.apps.BaseConfig'
