# Generated by Django 2.2.4 on 2019-08-30 04:27

from django.db import migrations


class Migration(migrations.Migration):

    dependencies = [
        ('document', '0017_remove_documenttemplate_citation_styles'),
        ('style', '0007_auto_20190811_1204'),
    ]

    operations = [
        migrations.DeleteModel(
            name='CitationLocale',
        ),
        migrations.DeleteModel(
            name='CitationStyle',
        ),
    ]
