.dk-select {
    background: -moz-linear-gradient(top, #fdfdfe, #eaeaea);
    background: -webkit-gradient(linear, left top, left bottom, from(#fdfdfe), to(#eaeaea));
    line-height: 34px;
    height: 36px;
}
.dk-select:focus {
    outline: 0;
}
.dk-select a {
    cursor: pointer;
    text-decoration: none;
}
/* Opens the dropdown and holds the menu label */
.dk-selected {
    overflow: hidden;
    white-space: nowrap;
    background-repeat: no-repeat;
    border: 1px solid #ccc;
    padding: 0 4px;
    height: 34px;
}

.dk-select-open-down .dk-selected {
    border-color: #ccc;
}

.dk-select-open-up .dk-selected:after, .dk-select-open-down .dk-selected:after {
    border-left-color: #ccc;
}
.dk-select-open-up .dk-selected:before, .dk-select-open-down .dk-selected:before {
    border-bottom-color: #ccc;
}
/* The outer container of the options */
.dk-select-open-down .dk-select-options {
    border-color: #ccc;
}

.dk-option {
    background-color: #fff;
    border-bottom: 1px solid #ccc;
    padding: 0 4px;
    height: 24px;
    line-height: 24px;
}
.dk-select-options .dk-option-highlight, .dk-select-options .dk-option-selected {
    background-color: #e0e0e0;
}
/* Inner container for options, this is what makes the scrollbar possible. */
.dk-select-options_inner {
    border: 1px solid #ccc;
    border-top-width: 0;
    border-bottom-width: 0;
    list-style: none;
    padding: 0;
}
/* Set a max-height on the options inner */
.dk-select-options_inner, .dk_touch .dk-select-options {
    max-height: 185px;
}

.dk-selected:hover, .dk-selected:focus {
  outline: none;
  border-color: #b73717;
}

 .dk-selected:hover:before, .dk-selected:focus:before {
     border-top-color: #b73717;
 }

.dk-selected:hover:after, .dk-selected:focus:after {
    border-left-color: #b73717;
}
